package org.nakedobjects.nof.util.memento;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import org.nakedobjects.noa.NakedObjectRuntimeException;


public class BinaryTransferableWriter extends AbstractTransferableWriter {
    private DataOutputStream dataOutputStream;
    private ByteArrayOutputStream byteArray;

    public BinaryTransferableWriter() {
        byteArray = new ByteArrayOutputStream();
        dataOutputStream = new DataOutputStream(byteArray);
    }

    public void writeInt(final int i) {
        try {
            dataOutputStream.writeInt(i);
        } catch (IOException e) {
            throw new TransferableException(e);
        }
    }

    public void writeString(final String string) {
        try {
            dataOutputStream.writeUTF(string);
        } catch (IOException e) {
            throw new TransferableException(e);
        }
    }

    public byte[] getBinaryData() {
        byte[] bytes = byteArray.toByteArray();
        try {
            dataOutputStream.close();
        } catch (IOException e) {
            throw new TransferableException(e);
        }
        return bytes;
    }

    public void writeLong(final long value) {
        try {
            dataOutputStream.writeLong(value);
        } catch (IOException e) {
            throw new TransferableException(e);
        }
    }

    public void close() {
        try {
            dataOutputStream.close();
            byteArray.close();
        } catch (IOException e) {
            throw new NakedObjectRuntimeException();
        }
    }

}
// Copyright (c) Naked Objects Group Ltd.
