package org.nakedobjects.nof.util.memento;

import java.util.Enumeration;
import java.util.Hashtable;

import org.nakedobjects.noa.adapter.Oid;
import org.nakedobjects.nof.core.util.DebugString;


class ObjectData extends Data {
    private static final long serialVersionUID = 7121411963269613347L;
    private final static Transferable NO_ENTRY = new Null();
    private final Hashtable fields = new Hashtable();

    public ObjectData(final Oid oid, final String resolveState, final String className) {
        super(oid, resolveState, className);
    }

    public ObjectData(final TransferableReader data) {
        super(data);

        int size = data.readInt();
        for (int i = 0; i < size; i++) {
            String key = data.readString();
            String type = data.readString();
            if (type.equals("O")) {
                Transferable object = data.readObject();
                fields.put(key, object);
            } else {
                String value = data.readString();
                fields.put(key, value);
            }
        }
    }

    public void addField(final String fieldName, final Object entry) {
        if (fields.containsKey(fieldName)) {
            throw new IllegalArgumentException("Field already entered " + fieldName);
        }
        fields.put(fieldName, entry == null ? NO_ENTRY : entry);
    }
    
    public void debug(DebugString debug) {
        super.debug(debug);
        for (Enumeration f = fields.keys(); f.hasMoreElements();) {
            String key = (String) f.nextElement();
            Object value = fields.get(key);

            debug.appendln(key, value);
            
            // TODO recurse!
        }        
    }

    public void writeData(final TransferableWriter data) {
        super.writeData(data);

        data.writeInt(fields.size());
        for (Enumeration f = fields.keys(); f.hasMoreElements();) {
            String key = (String) f.nextElement();
            Object value = fields.get(key);

            data.writeString(key);
            if (value instanceof Data || value instanceof Null) {
                data.writeString("O");
                data.writeObject((Transferable) value);
            } else {
                data.writeString("S");
                data.writeString((String) value);
            }
        }
    }

    public Object getEntry(final String fieldName) {
        Object entry = fields.get(fieldName);
        return entry == null || entry.getClass() == NO_ENTRY.getClass() ? null : entry;
    }

    public String toString() {
        return fields.toString();
    }

    public boolean containsField() {
        return fields != null && fields.size() > 0;
    }
}
// Copyright (c) Naked Objects Group Ltd.
