package org.nakedobjects.nof.util.xmlsnapshot;

import org.nakedobjects.nof.util.memento.Transferable;
import org.nakedobjects.nof.util.memento.TransferableWriter;


/**
 * <pre>
 *       [[95I]]
 *       [[abc]]
 *       [[95J][abc]]
 *       [[95J][[abc][89I]][23J]]
 * </pre>
 */
final class InlineTransferableWriter implements TransferableWriter {

    public final static String SEQUENCE_START = "{";
    public final static String SEQUENCE_END = "}";

    private final StringBuffer buffer;
    private String asString;

    InlineTransferableWriter() {
        buffer = new StringBuffer();
        buffer.append(SEQUENCE_START);
    }

    public void writeInt(final int i) {
        buffer.append(SEQUENCE_START).append(i).append("I").append(SEQUENCE_END);
    }

    public void writeString(final String str) {
        buffer.append(SEQUENCE_START).append(str).append(SEQUENCE_END);
    }

    public void writeLong(final long l) {
        buffer.append(SEQUENCE_START).append(l).append("J").append(SEQUENCE_END);
    }

    /**
     * Recursively invoke another instance of InlineTransferableWriter.
     */
    public void writeObject(final Transferable t) {
        InlineTransferableWriter itw = new InlineTransferableWriter();
        t.writeData(itw);
        itw.close();
        buffer.append(itw.toString());
    }

    /**
     * Appends child OID element to parent
     */
    public void close() {
        buffer.append(SEQUENCE_END);
        this.asString = buffer.toString();
    }

    /**
     * returns null until closed.
     */
    public String toString() {
        return asString;
    }
}
// Copyright (c) Naked Objects Group Ltd.
